const fh = require('os-file-handler');
const tools = require('os-tools');

const self = module.exports = {

    /**
     * Will unpack a series of apks, located in a path, to a desired destination
     */
    unpackApks: async function (jadxExePath, apksPath, destPath) {
        const tools = require('os-tools');
        fh.createDir(destPath);
        let apksList = fh.getDirContent(apksPath);

        for (let i = 0; i < apksList.length; i++) {
            console.log("unpacking " + apksList[i]);
            let stripped = fh.stripExtension(apksList[i]);
            if (stripped === '') {
                continue
            }
            let apkDestPath = fh.joinPath(apksPath, apksList[i])
            let currDestPath = fh.joinPath(destPath, stripped);

            fh.createDir(currDestPath);
            let cmd = jadxExePath + ' -d ' + tools.toStringRepresenation(currDestPath) + ' ' + tools.toStringRepresenation(apkDestPath);
            await tools.runCmd(cmd)
        }
    },


    /**
     * Will download a bunch of apks from a specific developer link in google play.
     * To download all of the apks from a specific developer, set a link to the developer page in
     * Google Play (for example: https://play.google.com/store/apps/developer?id=Remotify)
     *
     * @param downloadPath -> the path to which the apks will be downloaded
     * @param googlePlayDeveloperLink -> a link to a developer page in google play
     * @param downloadWaitTime -> how long to wait until each apk will be downloaded
     * @param dirForEachApk -> set to true if you would like each apk to be downloaded to a separate
     * dir
     * @param includePackageNameInApk -> set to true if you want to add the package name of the apk
     * in the apk name
     */
    downloadApks: async function (downloadPath,
                                  googlePlayDeveloperLink,
                                  downloadWaitTime,
                                  dirForEachApk = false) {

        // create the page
        const ph = require('os-puppeteer-helper');
        let br = await ph.createBrowser(googlePlayDeveloperLink);
        let page = br[1];

        let appsArr = await buildAppsArr(ph, page);
        console.log("found: " + appsArr.length + " apps");
        await iniDownload(ph, page, appsArr, downloadPath, downloadWaitTime, dirForEachApk);
        await ph.close(br[0]);
        return appsArr
    },

    /**
     * Will download a specific apk from ApkPure.com. Notice you should set the name of the app and
     * the package name exactly as listed in the google play.
     *
     * @param downloadPath -> the path to which the apk will be downloaded
     * @param appName -> the app's name
     * @param packageName -> the app's package name
     * @param downloadWaitTime -> how long to wait until the apk will be downloaded
     * @param dirForEachApk -> set to true if you would like each apk to be downloaded to a separate
     * @param includePackageNameInApk -> set to true if you want to add the package name of
     * the apk in the apk name
     */
    downloadApk: async function (downloadPath,
                                 appName,
                                 packageName,
                                 downloadWaitTime,
                                 dirForEachApk = false) {

        const ph = require('os-puppeteer-helper');
        let br = await ph.createBrowser();
        let app = {'name': appName, 'packageName': packageName};
        await iniDownload(ph, br[1], [app], downloadPath, downloadWaitTime, dirForEachApk);
        await ph.close(br[0]);
    }

};


// will init a download of a specific apk from ApkPure.com
async function iniDownload(ph, page, appsArr, downloadPath, downloadWaitTime, dirForEachApk) {
    let outputPath = downloadPath;
    if (!dirForEachApk) {
        await ph.setDownloadedFilesLocation(page, outputPath);
    }
    await fh.createDir(outputPath);

    var tools = require('os-tools');
    for (let i = 0; i < appsArr.length; i++) {
        var modifiedAppName = tools.replaceAll(appsArr[i]['name'], ' ', '-');
        var packageName = appsArr[i]['packageName'];
        if (dirForEachApk) {
            outputPath = await fh.joinPath(downloadPath, packageName);
            await fh.createDir(outputPath);
            await ph.setDownloadedFilesLocation(page, outputPath);
        }
        let url = 'https://apkpure.com/' + modifiedAppName + '/' + packageName + '/download?from=details';
        await ph.navigateTo(page, url, null, "div.copyright");
        let cantDownloadSelector = await ph.getElement(page, "#region-submit");
        if (cantDownloadSelector === undefined || cantDownloadSelector === null) {
            await tools.delay(downloadWaitTime)
        } else {
            await tools.promptUser("A problematic apk. Click download manually and inform me when you're done (done)")
        }

        console.log("done downloading: " + tools.toStringRepresenation(appsArr[i]))
    }
}

// will return the apps dict contains the names and package names from a developer link in the google play
async function buildAppsArr(ph, page) {

    await ph.waitForSelectorWithText(page, "span", "©2019 Google");

    // get all of the divs with the href
    let divs = await ph.getElements(page, "a[href^='/store/apps/details?id='] > div");

    let appsArr = [];
    for (let i = 0; i < divs.length; i++) {
        let name = await ph.getInnerText(page, divs[i]);

        const a = await ph.getParent(page, divs[i]);

        const href = await ph.getAttributeValueFromElement(a, 'href');
        const packageName = href.substring(href.indexOf('?id=') + 4);

        var app = {
            'name': name,
            'packageName': packageName
        };

        appsArr.push(app)
    }

    return appsArr
}
